@@ -594,16 +594,17 @@ print.manova.mvgls <- function(x, digits = max(3L, getOption("digits") - 3L), ..
     if(x$type=="II") cat("Type II MANOVA Tests:",x$test,"test statistic","\n")
     if(x$type=="III") cat("Type III MANOVA Tests:",x$test,"test statistic","\n")
     if(x$type=="glh") cat("General Linear Hypothesis Test:",x$test,"test statistic","\n")
+    if(x$type=="glhrm") cat("General Linear Hypothesis Test (repeated measures design):",x$test,"test statistic","\n")
     
     signif <- sapply(x$pvalue, function(i) if(i<0.001){"***"}else if(i<0.01){
       "**"}else if(i<0.05){"*"}else if(i<0.1){"."}else{""})
     
     table_results <- data.frame(Df=x$Df, stat=x$stat, approxF=x$approxF, numDf=x$NumDf, denDf=x$DenDf, pval=x$pvalue, signif=signif)
-    if(x$type!="glh") rownames(table_results) <- x$terms else rownames(table_results) <- "Contrasts L"
+    if(x$type!="glh" & x$type!="glhrm") rownames(table_results) <- x$terms else rownames(table_results) <- "Contrasts L"
     colnames(table_results) <- c("Df", "test stat", "approx F", "num Df", "den Df", "Pr(>F)", "")
     print(table_results, digits = digits, ...)
     cat("---","\n")
-    cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1")
+    cat("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1","\n")
     
     
     
@@ -613,12 +614,12 @@ print.manova.mvgls <- function(x, digits = max(3L, getOption("digits") - 3L), ..
     if(x$type=="II") cat("Type II MANOVA Tests with",x$nperm,"permutations:",x$test,"test statistic","\n")
     if(x$type=="III") cat("Type III MANOVA Tests with",x$nperm,"permutations:",x$test,"test statistic","\n")
     if(x$type=="glh")  cat("General Linear Hypothesis Test with",x$nperm,"permutations:",x$test,"test statistic","\n")
- 
+    if(x$type=="glhrm")  cat("General Linear Hypothesis Test (repeated measures design) with",x$nperm,"permutations:",x$test,"test statistic","\n")
     signif <- sapply(x$pvalue, function(i) if(i<0.001){"***"}else if(i<0.01){
       "**"}else if(i<0.05){"*"}else if(i<0.1){"."}else{""})
     
     table_results <- data.frame(stat=x$stat, pval=x$pvalue, signif=signif)
-    if(x$type!="glh") rownames(table_results) <- x$terms else rownames(table_results) <- "Contrasts L"
+    if(x$type!="glh" & x$type!="glhrm") rownames(table_results) <- x$terms else rownames(table_results) <- "Contrasts L"
     colnames(table_results) <- c("Test stat", "Pr(>Stat)", "")
     print(table_results, digits = digits, ...)
     cat("---","\n")
@@ -791,7 +792,7 @@ predict.mvgls <- function(object, newdata, ...){
     # If error=TRUE, we add it to the covariance matrix here
     #if(!is.na(object$mserr)) diag(V) = diag(V) + object$mserr
     
-    # Build the covariance matrix
+    # Build the covariance matrices
     w <- V[sp_name, train_sample, drop=FALSE]
     Vt <- V[train_sample, train_sample, drop=FALSE]
     
@@ -832,3 +833,99 @@ predict.mvgls <- function(object, newdata, ...){
     return(phy)
 }
 
+# ------------------------------------------------------------------------- #
+# print option for effect/association of multivariate tests                 #
+# options: x, digits, ...                                                   #
+#                                                                           #
+# ------------------------------------------------------------------------- #
+
+print.effects.mvgls <- function(x, digits = max(3L, getOption("digits") - 3L), ...){
+    
+    if(x$adjusted & x$parametric==TRUE){
+        tatsuoka <- attr(x$adjusted, "tatsuoka")
+        cat("## Multivariate measure(s) of association ##","\n")
+        if(tatsuoka) cat("## Tatsuoka' bias adjustment              ##","\n") else cat("## Serlin' bias adjustment                ##","\n")
+        print(x$effect, digits=digits)
+    }else{
+        cat("## Multivariate measure(s) of association ##","\n")
+        print(x$effect, digits=digits)
+        if(x$adjusted) cat("## Note: bias is empirically adjusted     ##","\n")
+    }
+    if(any(x$effect<0)) message("## Values < 0 represent no association    ##","\n")
+}
+
+# ------------------------------------------------------------------------- #
+# ancestral.mvgls                                                           #
+# options: object, ...                                                      #
+#                                                                           #
+# ------------------------------------------------------------------------- #
+
+
+## S3 Method for ancestral states estimation
+ancestral <- function(object, ...) UseMethod("ancestral")
+
+# core function
+ancestral.mvgls <- function(object, ...){
+    
+    # arguments
+    args <- list(...)
+    
+    # extract objects
+    if(!inherits(object,"mvgls")){
+        
+        # wrapper to "estim"
+        if(is.null(args[["data"]]) | is.null(args[["tree"]])) stop("Need a \"tree\" object and a new \"data\" matrix to predict ancestral states. See ?estim")
+        estim(tree = args$tree, data = args$data, object = object, asr=TRUE)
+        
+    }else if(inherits(object,"mvgls")){
+        
+        # If regression, must provide a regressor for the ancestral (nodes) states
+        # check if newdata is provided
+        if(any(object$dims$assign>0)){
+            
+            if(is.null(args[["na.action"]])) na.action <- na.pass else na.action <- args$na.action
+            if(is.null(args[["newdata"]])) stop("Regression model. You must provide a new \"dataset\" of predictors for each nodes. See also ?predict")
+            
+            Terms <- delete.response(object$terms)
+            # as in "stats v3.3.0"
+            m <- model.frame(Terms, args$newdata, xlev = object$xlevels, na.action = na.action)
+            
+            # check the arguments
+            if(!is.null(cl <- attr(Terms, "dataClasses"))) .checkMFClasses(cl, m)
+            X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
+            predicted_fit <- X %*% object$coefficients
+            
+        }else{
+            # Just use the grand mean - i.e. the ancestral states at the root
+            predicted_fit <- object$variables$X %*% object$coefficients
+            predicted_fit <- predicted_fit[-1,,drop=TRUE]
+        }
+        
+        # start estimating ancestral states using GLS
+        n <- object$dims$n
+        p <- object$dims$p
+        
+        # covariance for the nodes
+        if(!is.null(object$corrSt$diagWeight)){
+            V<-.Call("mvmorph_covar_ou_fixed", A=.vcvPhyloInternal(object$variables$tree), alpha=as.double(object$param), sigma=1, PACKAGE="mvMORPH")
+        }else{
+            V <- .vcvPhyloInternal(object$corrSt$phy)
+        }
+        indice <- (1:n)
+        AY <- V[-indice,indice]
+        vY <- V[indice,indice]
+        
+        # states at the nodes
+        residuals_fit <- object$residuals
+        recons_t <- (AY%*%pseudoinverse(vY)%*%residuals_fit)+predicted_fit
+        colnames(recons_t) = colnames(object$variables$Y)
+        rownames(recons_t) = paste("node_",n+1:Nnode(object$variables$tree), sep="")
+        #class(recons_t) = "anc.mvgls"
+        return(recons_t)
+        
+    }else{
+        stop("only works with \"mvgls\" class objects. See ?mvgls, or use instead \"estim\" function")
+    }
+    
+}
+
